package vishnu.server;

import java.io.*;
import java.util.*;
import vishnu.server.Search.*;
import vishnu.util.*;
import vishnu.datablock.*;

public class CKTester
{

    public static void main(String[] args){
	CKTester ckt = new CKTester(args);
    }

    
    public CKTester(String[] args){
	
	Vector docNum = new Vector();
	Vector descriptions = new Vector();
	
	String query = args[0];
	String coll = args[1];
	String vis_home = args[2];
	String view = null;
	if (args.length >= 4) {
	    view = args[3];
	}

	/**** read in ck words, doc freq and term freq ****/

	FileInputStream fis = null;
	ObjectInputStream in = null;
	
	String root = vis_home;
	if (!root.endsWith(File.separator)) {
	    root += File.separator;
	}
	root += "collections" + File.separator+ coll + File.separator + "index" + File.separator + "ck_index" + File.separator;
	if (view != null) {
	    root += view+File.separator;
	}
	String fileDocFreq = "doc_freq";
	String fileTermFreq = "term_freq";
	String fileCKWords = "ck_words";
	
	float[] docFreq = (float[])getObject(root+fileDocFreq);
	String[] ck_words = (String[])getObject(root + fileCKWords);
	Vector termFreq = (Vector)getObject(root + fileTermFreq);


	/**** get relevant docs ****/

	LUCSearcher lus = new LUCSearcher(coll, view); // coll should be fullpath
	//lus.setRootDirectory(vis_home);
	lus.search(query);
	docNum = lus.getDocIdentifiers();
	descriptions = lus.getDocDescriptions();
	
	int hits = docNum.size();
	

	/**** pool the candidate keywords and note their frequencies ****/
	
	Hashtable keyHash = new Hashtable();
	
	int count_ck_all = 0;

	for( int i = 0; i < hits; i++ ){
	    
	    int doc_index = ((Integer)docNum.elementAt(i)).intValue();
	    Hashtable ht = (Hashtable)termFreq.elementAt(doc_index);
	    count_ck_all += ht.size();
	    
	    Enumeration enumer = ht.keys();
	    while( enumer.hasMoreElements() ){
		Integer term_index = (Integer)enumer.nextElement();
		
		Integer set_freq;
		if( (set_freq = (Integer)keyHash.get(term_index)) == null ){
		    keyHash.put(term_index,new Integer(1));
		}else{
		    keyHash.put(term_index,new Integer(set_freq.intValue()+1));
		}
	    }
	}

	System.out.println("all keywords: " + count_ck_all);
	System.out.println("pooled: " + keyHash.size());


	/**** compute interesting weight for each keyword ****/

	Enumeration enumer = keyHash.keys();

	float[] weight_ck = new float[keyHash.size()];
	Integer[] weight_index = new Integer[keyHash.size()];
	int count = 0;

	while( enumer.hasMoreElements() ){
	    Integer index_ck = (Integer)enumer.nextElement();
	    Integer set_freq = (Integer)keyHash.get(index_ck);
	    
	    float set_freq_f = (float)set_freq.intValue()/hits;

	    float f1 = (float)set_freq_f/docFreq[index_ck.intValue()];
	    float f2 = -(float)Math.log(set_freq_f);

	    weight_index[count] = index_ck;
	    weight_ck[count] = - f1 * f2;

	    //System.out.println(set_freq_f+" "+docFreq[index_ck.intValue()]+" "+weight_ck[count]);
	    count++;
	}
	
	Misc.quickSort(weight_ck,weight_index);


	/**** store indices of up to 100 top ranked keywords ****/

	int wordCount = Math.min(100,count);
	int[] indices_kw = new int[wordCount];
	
	for( int i = 0; i < wordCount; i++ ){
	    indices_kw[i] = weight_index[i].intValue();
	    
	    float weight = -weight_ck[i];
	    String ck = ck_words[indices_kw[i]];
	    System.out.println(indices_kw[i] + " " + ck + " " + weight + " ");
	}


	/**** sort the indices_kw array in increasing order of indices ****/

	Misc.quickSort(indices_kw);


	/**** build full matrix: rows indices_kw (wordCount), col docNum (hits) ****/

	float[][] matrix_kd = new float[hits][wordCount];
	float[] sum_rows = new float[hits];
	int emptyRows = 0;

	for( int i = 0; i < hits; i++ ){
	    
	    int doc_index = ((Integer)docNum.elementAt(i)).intValue();
	    Hashtable ht = (Hashtable)termFreq.elementAt(doc_index);
	    
	    for( int j = 0; j < wordCount; j++ ){
		
		Integer term_freq;
		
		if( (term_freq = (Integer)ht.get(new Integer(indices_kw[j]))) != null ){
		    float w = -(float)(Math.log(1+term_freq.intValue()) * Math.log(docFreq[indices_kw[j]]));
		    matrix_kd[i][j] = w;
		    sum_rows[i] += w*w;
		}
		System.out.print(matrix_kd[i][j] + " ");
	    }
	    
	    System.out.println();
	    

	    /**** sqrt sum to obtain |z| ****/

	    if( sum_rows[i] != 0 ) sum_rows[i] = (float)Math.sqrt(sum_rows[i]); else emptyRows++;
	}
	
	/**** populate sparseMatrix dividing each entry by the row_sum ****/
	
	SparseMatrix matrix = new SparseMatrix(hits-emptyRows,wordCount);

	int ci = 0;
	
	for( int i = 0; i < hits; i++ ){
	    
	    if( sum_rows[i] == 0 )
		continue;
	    
	    for( int j = 0; j < wordCount; j++ ){
		
		float m = matrix_kd[i][j];
		
		if( m != 0 ){
		    float norm_m = (float)m/sum_rows[i];
		    matrix.addDataItem(ci, j, norm_m);
		    System.out.print(j + " (" + norm_m + ") ");
		}
	    }

	    ci++;
	    System.out.println();
	}
    }
    


    private Object getObject(String str){
	Object o = null;
	FileInputStream fis = null;
	ObjectInputStream in = null;
	try{
	    fis = new FileInputStream(str);
	    in = new ObjectInputStream(fis);
	    o = in.readObject();
	    in.close();
	}
	catch(IOException ex){
	    ex.printStackTrace();
	}
	catch( ClassNotFoundException ex){
	    ex.printStackTrace();
	}
	return o;
    }
}


class Node{
    Node next = null;
    float value = -1;
    int docIndex = -1;
    int kwIndex = -1;

    public Node(int d, int k)
    {
	docIndex = d;
	kwIndex = k;
    }
}
